<?php

require './cos-php-sdk-v5/vendor/autoload.php';

use Qcloud\Cos\Client;

function main_handler($event, $context) {
    $cos = new Client([
        'region'        => getenv('cos_region'),
        'schema'        => 'https',
        'credentials'   => [
            'secretId'  => getenv('cos_secret_id'),
            'secretKey' => getenv('cos_secret_key'),
        ],
    ]);

    $bucket = getenv('cos_bucket');

    if(getenv('cdn_domain')) {
        $cdn_domain = sprintf('https://%s/', getenv('cdn_domain'));
    } else {
        $cdn_domain = sprintf('https://%s.cos.%s.myqcloud.com/', $bucket, getenv('cos_region'));
    }

    $event = (array) $event;

    $query_string = (array) $event['queryString'];

    if(isset($query_string['type'])) {
        if($query_string['type'] == 'sign') {
            $path = get_path(sprintf('files/%s.ipa', str_rand()));

            try {
                $sign = $cos->getPresignetUrl('putObject', [
                            'Bucket'        => $bucket,
                            'Key'           => $path,
                            'Body'          => '',
                        ], '+10 minutes');

                return result([
                    'sign_url'  => (string) $sign,
                    'cdn_url'   => 'https://' . $sign->getHost() . $sign->getPath(),
                ]);
            } catch (\Exception $e) {
                return error('获取签名失败');
            }
        } else if($query_string['type'] == 'create') {
            if(!($query_string['name'] && $query_string['version'] && $query_string['url'] && $query_string['bundle_identifier'])) {
                return error();
            }

            $url = parse_url($query_string['url']);

            try {
                $path = get_path(sprintf('plist/%s.plist', str_rand()));

                $result = $cos->putObject([
                    'Bucket'        => $bucket,
                    'Key'           => $path,
                    'Body'          => generated_plist($cdn_domain . $url['path'], $query_string),
                    'ContentType'   => 'text/xml',
                ]);

                return result([
                    'cdn_url'   => $cdn_domain . $path,
                ]);
            } catch (\Exception $e) {
                return error('上传plist失败');
            }
        }
    }

    return [
        'isBase64Encoded'   => false,
        'statusCode'        => 200,
        'headers'           => [
            'Content-Type' => 'text/html',
        ],
        'body'              => file_get_contents('./index.html'),
    ];
}

function str_rand(int $length = 6){
    return bin2hex(random_bytes(($length - ($length % 2)) / 2));
}

function generated_plist($ipa_path, array $body) {
    return '<?xml version="1.0" encoding="UTF-8"?>
        <!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
        <plist version="1.0">
        <dict>
            <key>items</key>
            <array>
                <dict>
                    <key>assets</key>
                    <array>
                        <dict>
                            <key>kind</key>
                            <string>software-package</string>
                            <key>url</key>
                            <string>' . $ipa_path . '</string>
                        </dict>
                    </array>
                    <key>metadata</key>
                    <dict>
                        <key>bundle-identifier</key>
                        <string>' . $body['bundle_identifier'] . '</string>
                        <key>bundle-version</key>
                        <string>' . $body['version'] . '</string>
                        <key>kind</key>
                        <string>software</string>
                        <key>title</key>
                        <string>' . $body['name'] . '</string>
                    </dict>
                </dict>
            </array>
        </dict>
        </plist>';
}

function get_path(string $name) {
    return getenv('storage_prefix') . $name;
}

function result($data = []) {
    return [
        'isBase64Encoded'   => false,
        'statusCode'        => 200,
        'headers'           => [
            'Content-Type' => 'application/json',
        ],
        'body'              => json_encode([
            'code'          => 1,
            'data'          => $data,
        ]),
    ];
}

function error($message = '参数错误') {
    return [
        'isBase64Encoded'   => false,
        'statusCode'        => 200,
        'headers'           => [
            'Content-Type' => 'application/json',
        ],
        'body'              => json_encode([
            'code'          => 0,
            'message'       => $message,
        ]),
    ];
}
